/*
**	BPALogin - lightweight portable BIDS2 login client
**	Copyright (c) 2001-3 Shane Hyde, and others.
** 
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
** 
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
** 
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
*/ 

#include "bpalogin.h"

static void sleeps(int seconds)
{
	struct timeval timo;

	timo.tv_sec = seconds;
	timo.tv_usec = 0;
	select(1, NULL, NULL, NULL, &timo);
}		

/*
 **  Main processing loop.  Logs in, handles heartbeats, and logs out on request
 **
 **  Returns - 0 - We are no longer connected, and should not retry.
 **            1 - We are no longer connnected and should retry.
 */
int mainloop(struct session *s)
{
	socklen_t addrsize;
	struct hostent *host;
	struct sockaddr_in listenaddr;	

	s->listensock = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
	s->localaddr.sin_family = AF_INET;
	s->localaddr.sin_port = htons(s->localport);
	if (s->localaddress[0] != '\0') {
		host = gethostbyname(s->localaddress);

		if (host != NULL && host->h_addrtype == AF_INET)
			s->localaddr.sin_addr.s_addr = *((unsigned int*)(host->h_addr_list[0]));
		else
			s->localaddr.sin_addr.s_addr = inet_addr(s->localaddress);
	} else {
		s->localaddr.sin_addr.s_addr = INADDR_ANY;
	}

	addrsize = sizeof(struct sockaddr_in);
	bind(s->listensock, (struct sockaddr *)&s->localaddr, sizeof(s->localaddr));
	getsockname(s->listensock, (struct sockaddr *)&listenaddr, &addrsize);

	s->listenport = ntohs(listenaddr.sin_port);
	s->sessionid = time(NULL);

	strcpy(s->osname, "WOKAO:2005~2008");
	strcpy(s->osrelease, "DNI-No.50967395");

	host = gethostbyname(s->authserver);
	if (host != NULL && host->h_addrtype == AF_INET)
		s->authhost.sin_addr.s_addr = *((unsigned int*)(host->h_addr_list[0]));
	else
		s->authhost.sin_addr.s_addr = inet_addr(s->authserver);
	s->authhost.sin_port = htons(s->authport);
	s->authhost.sin_family = AF_INET;

	DEBUGP("Auth Host = %s:%d\n", inet_ntoa(s->authhost.sin_addr), s->authport);
	DEBUGP("Listening on port %d\n", s->listenport);

	show_bpa_status(0);
	set_bpa_mode(s, 0, 0);

	if (bpa_login(s)) {
		if (handle_heartbeats(s)) {
			close(s->listensock);
			return 0;
		}

		/* Try to logout the server anyway~ */
		bpa_logout(0, s);

		if (s->linkmode != DEMAND)
			syslog(6, "[Internet disconnected]");

		close(s->listensock);

		if (s->linkmode == RUNONE) {
			/* stop DHCP-Client to drop all traffic ... */
			system("killall -SIGUSR2 udhcpc");
			system("killall udhcpc");
			return 0;
		}

		/* wait for a moment ... */
		sleeps(2);

		if (s->linkmode == DEMAND)
			bpa_demand(s);

		DEBUGP("Restart the authentication ...\n");
	} else {
		DEBUGP("Sleeping for 30 seconds after login failed...\n");

		close(s->listensock);

		sleeps(30);
	}

	return 1;
}

