#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>

#include "dnsmasq.h"

#ifdef DOME_LED_FUTURE
#define BLOCK_DOMAIN_NAME	"block.opendns.com"
#define DNS_HEADER_LEN		12

#ifdef DOME_DEBUG
#   define DBG(fmt, args...)   printf("[DOME_DNS] "fmt, ##args)
#else
#   define DBG(fmt, args...)   do{ }while(0)
#endif

extern int config_match(char *name, char *match);

typedef struct _dns_domain_t
{
	char name[255];
	u32 len;
} dns_domain_t;

typedef struct _dns_header_t
{
	u16 id;
	u16 flags;
	u16 ques;
	u16 ans_rrs;
	u16 auth_rrs;
	u16 addi_rrs;
	/*query */
} dns_header_t;

void parse_dns_domain(u8 * pkg, dns_domain_t * dom)
{
	char name[256] = { 0 };
	char *tmp, *p;
	int i, len;

	if (!pkg || !dom)
		return;

	tmp = pkg;
	p = name;

	len = pkg[0];
	for (i = 0; *tmp != '\0'; i++, tmp++) {
		if (i > 255)
			break;
		if (i == len + 1) {
			len = len + *tmp + 1;
			*p++ = '.';
		}
		else {
			if (i != 0)
				*p++ = *tmp;
		}
	}

	*p = '\0';

	strcpy(dom->name, name);

	dom->len = strlen(name) + 2 + 2 + 2;
}

u32 extract_dns_answer(u8 * pkg)
{
	u8 *tmp;
	dns_header_t *hdr_ptr;
	dns_domain_t dom;

	if (!pkg)
		return -1;

	hdr_ptr = (dns_header_t *) pkg;

	if (hdr_ptr->flags != htons(0x8180)) {
		DBG("Not query response skip... flags 0x%x\n", hdr_ptr->flags);
		return -1;
	}
	if (hdr_ptr->ques != htons(0x0001) || hdr_ptr->ans_rrs != htons(0x0001)) {
		/*Now only parse the simple situation */
		DBG("more than 1 ques and answ skip...ques = 0x%x ans_rrs = 0x%x\n", hdr_ptr->ques, hdr_ptr->ans_rrs);
		return -1;
	}

	tmp = pkg + DNS_HEADER_LEN;

	parse_dns_domain(tmp, &dom);
	DBG("domain %s len %d\n", dom.name, dom.len);

	if (strcmp(BLOCK_DOMAIN_NAME, dom.name) == 0)
		return 1;

	return 0;
}

int dome_led_blink_for_pc(u8 * pkg_reply)
{
	struct in_addr inaddr;
	char *cfg_ip;
	char *p;

	if (config_match("ParentalControl", "0") || !config_match("dome_led_func", "2"))
		return -1;

	return extract_dns_answer(pkg_reply);
}
#endif //DOME_LED_FUTURE
