#include <stdio.h>
#include <netdb.h>
#include <string.h>
#include <stdlib.h>
#include <getopt.h>
#include <ctype.h>

#include <iptables.h>

#include <linux/netfilter_ipv4/ipt_dnifilter.h>

static void
help(void)
{
    printf(
    "DNIFILTER options:\n"
    " --eva	For Netgear EVA traffic\n"
    " DEBUG SUPPPORT, use only if you know why\n"
    " --debug		Generate kernel debug output, THIS WILL SLOW DOWN THE FILTER\n"
    "\nExamples:\n"
    " iptables -t mangle -A FORWARD -m dnifilter --eva -j MARK --set-mark 0x01\n"
    );
}

static struct option opts[] = {
        { "eva", 0, 0, '1' },
	{ "debug", 0, 0, 'j' },
        {0}
};

	

static void
init(struct ipt_entry_match *m, unsigned int *nfcache)
{
    struct ipt_dnifilter_info *info = (struct ipt_dnifilter_info *)m->data;

    *nfcache |= NFC_UNKNOWN;

    /*init the module with default values*/
    info->cmd = 0;
    info->debug = 0;

}
	

static int
parse(int c, char **argv, int invert, unsigned int *flags,
	const struct ipt_entry *entry,
	unsigned int *nfcache,
	struct ipt_entry_match **match)
{
    struct ipt_dnifilter_info *info = (struct ipt_dnifilter_info *)(*match)->data;
    
    switch (c) {
	case '1':		/*cmd: eva*/
	    if ((*flags) != 0)
		    exit_error(PARAMETER_PROBLEM,
				"dnifilter: `--eva' may only be "
				"specified alone!");
	    if (invert) exit_error(PARAMETER_PROBLEM, "dnifilter: invert [!] is not allowed!");
	    *flags = DNIFILTER_EVA;
	    info->cmd = *flags;
	    break;
	    
	case 'j':		/*cmd: debug*/
	    if (invert) exit_error(PARAMETER_PROBLEM, "dnifilter: invert [!] is not allowed!");
	    info->debug = 1;
	    break;																											

	default:
            exit_error(PARAMETER_PROBLEM,
	    "\ndnifilter-parameter problem: for dnifilter usage type: iptables -m dnifilter --help\n");
	    return 0;
    }
    return 1;
}


static void
final_check(unsigned int flags)
{
    if (!flags)
            exit_error(PARAMETER_PROBLEM,
	    "\ndnifilter-parameter problem: for dnifilter usage type: iptables -m dnifilter --help\n");
}



static void
print(const struct ipt_ip *ip,
        const struct ipt_entry_match *match,
	int numeric)
{
    struct ipt_dnifilter_info *info = (struct ipt_dnifilter_info *)match->data;
    
    printf("dnifilter ");
    if (info->cmd  == DNIFILTER_EVA) printf(" --eva");
    if (info->debug != 0) printf(" --debug");
    printf(" ");
}
			    						   


static void
save(const struct ipt_ip *ip, const struct ipt_entry_match *match)
{
    struct ipt_dnifilter_info *info = (struct ipt_dnifilter_info *)match->data;
    
    printf("dnifilter ");
    if (info->cmd  == DNIFILTER_EVA) printf(" --eva");
    if (info->debug != 0) printf(" --debug");
}
		


static 
struct iptables_match dnifilter= 
{ 
    .next           = NULL,
    .name           = "dnifilter",
    .version        = IPTABLES_VERSION,
    .size           = IPT_ALIGN(sizeof(struct ipt_dnifilter_info)),
    .userspacesize  = IPT_ALIGN(sizeof(struct ipt_dnifilter_info)),
    .help           = &help,
    .init           = &init,
    .parse          = &parse,
    .final_check    = &final_check,
    .print          = &print,
    .save           = &save,
    .extra_opts     = opts
};
					    


void _init(void)
{
    register_match(&dnifilter);
}

